{% extends "layout.html" %}
{% block content %}
  <div class="container">
    <div class="row">
      <h2>Allow?</h2>
      <div class="col-md-12">
        <form method="POST">
          <button type="button" class="btn btn-warning" name="confirm" value="yes">Authorize</button>
          <button type="button" class="btn btn-danger" name="confirm" value="no">Deny</button>
        </form>
      </div>
    </div>
  </div>
{% endblock %}
